// x.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "windows.h"
#include "db.h"
#include "assert.h"

using namespace svp;

// this struct contain data
struct CData : public cdb_base_struct
{
	long k;
	std::wstring s;

	cdb_tstring GetKey(long lKeyNumber) const;
};

// must define
// key generator, generate key from items
cdb_tstring CData::GetKey(long lKeyNumber) const
{
	switch (lKeyNumber)
	{
	case 0 :
		return cdb_keygen::GetKey(k);
		break;
	case 1 :
		return cdb_keygen::GetKey(s, 5);
		break;
	case 2 :
		return cdb_keygen::GetKey(k) + cdb_keygen::GetKey(s, 5);
		break;
	}
	assert(false);
	return cdb_tstring();
};

int main(int argc, char* argv[])
{
	// create db

	cdb<CData> d(3);


	// put data to db

	CData s; 

	// push back

	s.k = 3;
	s.s = L"TEST3";

	d.push_back(s);

	s.k = 1;
	s.s = L"TEST1";

	d.push_back(s);

	s.k = 2;
	s.s = L"TEST2.1";

	d.push_back(s);

	s.k = 2;
	s.s = L"TEST2.2";

	d.push_back(s);

	// insert

	s.k = 2;
	s.s = L"TEST2.3";

	d.insert_at(1, s);

	// push front 

	s.k = 2;
	s.s = L"TEST2.4";

	d.push_front(s);

	s.k = 3;
	s.s = L"TEST3";

	d.push_back(s);

	
	// iterate, use as vector

	wprintf(L"use as vector : \n");

	for (long k = 0; k < d.size(); k++)
	{
		s = d[k];
		wprintf(L"%s ",s.s.data());
	}

	
	// find item by key

	wprintf(L"\n\nfind only one item, key 0 : \n");

	CData sf;
	sf.k = 1;
	sf.s = L"";
	assert(d.find(0, sf, s));	
	wprintf(L"%s\n",s.s.data());


	// find item by key, keys not unique

	wprintf(L"\nfind synonyms item, key 0 : \n");

	sf.k = 2;
	sf.s = L"";

	// print count of synonyms
	wprintf(L"synonyms count : %d\n", d.synonyms_count(0, sf));

	long lSyn = 0;
	while (d.find(0, sf, s, lSyn++))
	{
		wprintf(L"%s ",s.s.data());
	}

	
	// delete TEST2.3 item

	wprintf(L"\n\ndelete item TEST2.3\n");

	sf.k = -1;
	sf.s = L"TEST2.3";
	assert(d.erase(1, sf));
	for (k = 0; k < d.size(); k++)
	{
		s = d[k];
		wprintf(L"%s ",s.s.data());
	}

	
	// find item by key, use as map, key 1

	wprintf(L"\n\nfind only one item, key 1 : \n");

	sf.k = -1;
	sf.s = L"TEST1";
	// sf.s = L""; // generate assert, item not found
	assert(d.find(1, sf, s));	
	wprintf(L"%s %d\n",s.s.data(), s.k);

	
	// find item by key, use as map, key 2

	wprintf(L"\nfind only one item, key 2 : \n");

	sf.k = 1;
	sf.s = L"TEST1";
	// sf.k = -1; // generate assert, item not found
	// sf.s = L""; // generate assert, item not found
	assert(d.find(2, sf, s));	
	wprintf(L"%s %d\n",s.s.data(), s.k);

	// change

	wprintf(L"\nchange item TEST2.1\n");

	sf.k = -1;
	sf.s = L"TEST2.1";
	long pos = d.get_position(1, sf);
	d[pos].s = L"\"" + d[pos].s + L" - is changed ...\"";
	for (k = 0; k < d.size(); k++)
	{
		s = d[k];
		wprintf(L"%s ",s.s.data());
	}


	::MessageBoxW(NULL, L"Click to ok for program close.", L"", MB_ICONASTERISK);

	return 0;
}

